/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.db.rmi;

import java.rmi.RemoteException;
import java.util.Date;

import de.willuhn.datasource.rmi.DBObject;

/**
 * Interface of the business object for history. According to the SQL table, we
 * define some getter an setter here.
 * 
 * <pre>
 * CREATE TABLE history (
 *   id NUMERIC default UNIQUEKEY('history'),
 *   aktien_id int(4) NOT NULL,
 *   oderDate date,
 *   orderAmount float,
 *   orderPrice float,
 *   orderPriceInMainCurrency float,
 *   additionalCosts float,
 *   additionalCostsInMainCurrency float,
 *   additionalCostsCurrency varchar(5),
 *   comment text,
 *   UNIQUE (id),
 *   PRIMARY KEY (id)  
 * );
 * </pre>
 * 
 * <br>
 * Getters and setters for the primary key (id) are not needed. Every one of the
 * following methods has to throw a RemoteException. <br>
 */
public interface OrderHistory extends DBObject {

	/**
	 * Liefert das Wertpapier zu der Order.
	 * 
	 * @return das Wertpapier.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	Wertpapier getAktie() throws RemoteException;

	/**
	 * Liefert das OrderDate der Order
	 * 
	 * @return orderDate der Order.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */

	Date getOrderDate() throws RemoteException;

	/**
	 * Liefert die Anzahl der Aktien der Order.
	 * 
	 * @return orderAmount der Order.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	double getOrderAmount() throws RemoteException;

	/**
	 * Liefert den Aktieneinzelpreis der Order.
	 * 
	 * @return orderPrice der Order.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	double getOrderPrice() throws RemoteException;

	/**
	 * Liefert den Aktieneinzelpreis der Order in der Hauptw&auml;hrung.
	 * 
	 * @return orderPrice der Order.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	double getOrderPriceInMainCurrency() throws RemoteException;

	/**
	 * Liefert zustzliche Kosten der Order.
	 * 
	 * @return additionalCosts der Order.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */

	double getAdditionalCosts() throws RemoteException;

	/**
	 * Liefert zustzliche Kosten der Order in der Hauptw&auml;hrung.
	 * 
	 * @return additionalCosts der Order.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	double getAdditionalCostsInMainCurrency() throws RemoteException;

	/**
	 * Liefert die W&auml;hrung der zustzliche Kosten der Order.
	 * 
	 * @return currency der zustzliche Kosten der Order.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	String getAdditionalCostsCurrency() throws RemoteException;

	/**
	 * Setzt das Wertpapier der Order.
	 * 
	 * @param wertpapier
	 *            das Wertpapier
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	void setAktie(Wertpapier wertpapier) throws RemoteException;

	/**
	 * Setzt das Datum der OrderImpl.
	 * 
	 * @param orderDate
	 *            der OrderImpl.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	void setOrderDate(Date orderDate) throws RemoteException;

	/**
	 * Setzt die Aktienanzahl der OrderImpl.
	 * 
	 * @param orderAmount
	 *            die Aktienanzahl der OrderImpl. Negative Anzahl heit Verkauf.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	void setOrderAmount(double orderAmount) throws RemoteException;

	/**
	 * Setzt den Preis pro Wertpapier der OrderImpl.
	 * 
	 * @param orderPrice
	 *            den Preis pro Wertpapier der OrderImpl.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	void setOrderPrice(double orderPrice) throws RemoteException;

	/**
	 * Setzt den Preis pro Wertpapier der Order in der Hauptw&auml;hrung.
	 * 
	 * @param orderPrice
	 *            den Preis pro Wertpapier der Order in der Hauptw&auml;hrung.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	void setOrderPriceInMainCurrency(double orderPrice) throws RemoteException;

	/**
	 * Setzt die zustzlichen Kosten der Order.
	 * 
	 * @param additionalCosts
	 *            die zustzlichen Kosten der Order.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	void setAdditionalCosts(double additionalCosts) throws RemoteException;

	/**
	 * Setzt die zustzlichen Kosten der Order in der Hauptw&auml;hrung.
	 * 
	 * @param additionalCosts
	 *            die zustzlichen Kosten der Order in der Hauptw&auml;hrung.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	void setAdditionalCostsInMainCurrency(double additionalCosts)
			throws RemoteException;

	/**
	 * Setzt die W&auml;hrung der zustzliche Kosten der Order.
	 * 
	 * @param currency
	 *            der zustzliche Kosten der Order.
	 * 
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	void setAdditionalCostsCurrency(String currency) throws RemoteException;

	/**
	 * Liefert die Beschreibung der Order.
	 * 
	 * @return die Beschreibung der Order.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	String getComment() throws RemoteException;

	/**
	 * Setzt die Beschreibung der Order.
	 * 
	 * @param comment
	 *            die Beschreibung der Order.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	void setComment(String comment) throws RemoteException;

}
