/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.db.server;

import java.rmi.RemoteException;
import java.util.Date;

import de.frankmuenster.jameica.depot.db.rmi.OrderHistory;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.utils.I18N;
import de.willuhn.datasource.db.AbstractDBObject;
import de.willuhn.datasource.rmi.ObjectNotFoundException;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Implementation of the OrderHistory table database access.
 *
 * @author Frank Mnster
 */
public class OrderHistoryImpl extends AbstractDBObject implements OrderHistory {

    /**
     * Name for the stock_id column
     */
    private static final String STOCK_ID = "aktie_id"; //$NON-NLS-1$

    /**
     * Serial UID
     */
    private static final long serialVersionUID = 8548057261191049423L;

    /**
     * @throws RemoteException
     */
    public OrderHistoryImpl() throws RemoteException {
	super();
    }

    /**
     * @see OrderHistory#getAdditionalCosts()
     */
    @Override
    public double getAdditionalCosts() throws RemoteException {

	final Double additionalCosts = (Double) this.getAttribute("additionalCosts"); //$NON-NLS-1$
	return additionalCosts == null || Double.isNaN(additionalCosts) ? 0.0d : additionalCosts.doubleValue();
    }

    /**
     * @see OrderHistory#getAdditionalCostsCurrency()
     */
    @Override
    public String getAdditionalCostsCurrency() throws RemoteException {

	return (String) this.getAttribute("additionalCostsCurrency"); //$NON-NLS-1$
    }

    /**
     * @see OrderHistory#getAdditionalCostsInMainCurrency()
     */
    @Override
    public double getAdditionalCostsInMainCurrency() throws RemoteException {

	final Double acInMainCurrency = (Double) this.getAttribute("additionalCostsInMainCurrency"); //$NON-NLS-1$
	return acInMainCurrency == null || Double.isNaN(acInMainCurrency) ? 0.0d : acInMainCurrency.doubleValue();
    }

    /**
     * @see OrderHistory#getAktie()
     */
    @Override
    public Wertpapier getAktie() throws RemoteException {

	// Yes, we can cast this directly to Wertpapier, because
	// getForeignObject(String)
	// contains the mapping for this attribute.
	try {
	    return (Wertpapier) this.getAttribute(OrderHistoryImpl.STOCK_ID); // NOPMD
	} catch (final ObjectNotFoundException e) {
	    return null;
	}
    }

    /**
     * @see OrderHistory#getComment()
     */

    @Override
    public String getComment() throws RemoteException {

	return (String) this.getAttribute("comment"); //$NON-NLS-1$
    }

    /**
     * @see OrderHistory#getOrderAmount()
     */
    @Override
    public double getOrderAmount() throws RemoteException {

	final Double orderAmount = (Double) this.getAttribute("orderAmount"); //$NON-NLS-1$
	return orderAmount == null || Double.isNaN(orderAmount) ? 0.0d : orderAmount.doubleValue();
    }

    /**
     * @see OrderHistory#getOrderDate()
     */
    @Override
    public Date getOrderDate() throws RemoteException {

	return (Date) this.getAttribute("orderDate"); //$NON-NLS-1$
    }

    /**
     * @see OrderHistory#getOrderPrice()
     */
    @Override
    public double getOrderPrice() throws RemoteException {

	final Double ordnerprice = (Double) this.getAttribute("orderPrice"); //$NON-NLS-1$
	return ordnerprice == null || Double.isNaN(ordnerprice) ? 0.0d : ordnerprice.doubleValue();
    }

    /**
     * @see OrderHistory#getOrderPriceInMainCurrency()
     */
    @Override
    public double getOrderPriceInMainCurrency() throws RemoteException {

	final Double opInMainCurrency = (Double) this.getAttribute("orderPriceInMainCurrency"); //$NON-NLS-1$
	return opInMainCurrency == null || Double.isNaN(opInMainCurrency) ? 0.0d : opInMainCurrency.doubleValue();
    }

    /**
     * @see OrderHistory#getPrimaryAttribute()
     */
    @Override
    public String getPrimaryAttribute() throws RemoteException {

	return "id"; //$NON-NLS-1$
    }

    /**
     * @see OrderHistory#setAdditionalCosts(double)
     */
    @Override
    public void setAdditionalCosts(final double additionalCosts) throws RemoteException {

	this.setAttribute("additionalCosts", Double.valueOf(additionalCosts)); //$NON-NLS-1$

    }

    /**
     * @see OrderHistory#setAdditionalCostsCurrency(String)
     */
    @Override
    public void setAdditionalCostsCurrency(final String additionalCosts) throws RemoteException {

	this.setAttribute("additionalCostsCurrency", additionalCosts); //$NON-NLS-1$

    }

    /**
     * @see OrderHistory#setAdditionalCostsInMainCurrency(double)
     */
    @Override
    public void setAdditionalCostsInMainCurrency(final double additionalCosts) throws RemoteException {

	this.setAttribute("additionalCostsInMainCurrency", //$NON-NLS-1$
		Double.valueOf(additionalCosts));

    }

    /**
     * @see OrderHistory#setAktie(Wertpapier)
     */
    @Override
    public void setAktie(final Wertpapier wertpapier) throws RemoteException {

	// casting wird in foreignObject bernommen
	this.setAttribute(OrderHistoryImpl.STOCK_ID, wertpapier);
    }

    /**
     * @see OrderHistory#setComment(String)
     */

    @Override
    public void setComment(final String comment) throws RemoteException {

	this.setAttribute("comment", comment); //$NON-NLS-1$
    }

    /**
     * @see OrderHistory#setOrderAmount(double)
     */
    @Override
    public void setOrderAmount(final double orderAmount) throws RemoteException {

	this.setAttribute("orderAmount", Double.valueOf(orderAmount)); //$NON-NLS-1$

    }

    /**
     * @see OrderHistory#setOrderDate(Date)
     */
    @Override
    public void setOrderDate(final Date orderDate) throws RemoteException {

	this.setAttribute("orderDate", orderDate); //$NON-NLS-1$

    }

    /**
     * @see OrderHistory#setOrderPrice(double)
     */
    @Override
    public void setOrderPrice(final double orderPrice) throws RemoteException {

	this.setAttribute("orderPrice", Double.valueOf(orderPrice)); //$NON-NLS-1$

    }

    /**
     * @see OrderHistory#setOrderPriceInMainCurrency(double)
     */
    @Override
    public void setOrderPriceInMainCurrency(final double orderPrice) throws RemoteException {

	this.setAttribute("orderPriceInMainCurrency", Double.valueOf(orderPrice)); //$NON-NLS-1$

    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#getForeignObject(java.lang.String)
     */
    @Override
    protected Class<Wertpapier> getForeignObject(final String field) throws RemoteException {

	// the system is able to resolve foreign keys and loads
	// the according objects automatically. You only have to
	// define which class handles which foreign key.
	if (OrderHistoryImpl.STOCK_ID.equals(field)) {
	    return Wertpapier.class; // NOPMD
	}
	return null;
    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#getTableName
     */
    @Override
    protected String getTableName() {

	return "orderHistory"; //$NON-NLS-1$
    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#insertCheck()
     */
    @Override
    @SuppressWarnings({ "PMD.ModifiedCyclomaticComplexity", "PMD.StdCyclomaticComplexity", "PMD.CyclomaticComplexity" })
    protected void insertCheck() throws ApplicationException {

	try {
	    if (this.getOrderDate() == null) {
		throw new ApplicationException(I18N.tr("OrderHistoryImpl.enterDate")); //$NON-NLS-1$
	    }

	    else if (Double.isNaN(this.getOrderAmount())) {
		throw new ApplicationException(I18N.tr("OrderHistoryImpl.enterAmount")); //$NON-NLS-1$
	    }

	    else if (Double.isNaN(this.getOrderPrice())) {
		throw new ApplicationException(I18N.tr("OrderHistoryImpl.enterPrice")); //$NON-NLS-1$
	    }

	    else if (this.getAktie() == null) {
		throw new ApplicationException(I18N.tr("OrderHistoryImpl.chosseStock")); //$NON-NLS-1$
	    }

	    else if (this.getAktie().isNewObject()) {
		throw new ApplicationException(I18N.tr("OrderHistoryImpl.storeFirst")); //$NON-NLS-1$
	    }

	    if (Double.isNaN(this.getOrderPriceInMainCurrency())) {
		this.setOrderPriceInMainCurrency(0.0d);
	    }

	    if (Double.isNaN(this.getAdditionalCosts())) {
		this.setAdditionalCosts(0.0d);
	    }

	    if (Double.isNaN(this.getAdditionalCostsInMainCurrency())) {
		this.setAdditionalCostsInMainCurrency(0.0d);
	    }

	    if (this.getAdditionalCostsCurrency() == null) {
		this.setAdditionalCostsCurrency(""); //$NON-NLS-1$
	    }

	} catch (final RemoteException e) {
	    Logger.error(I18N.tr("OrderHistoryImpl.insertCheckFaild"), e); //$NON-NLS-1$
	    throw (ApplicationException) new ApplicationException(I18N.tr("OrderHistoryImpl.unableToStrore")) //$NON-NLS-1$
		    .initCause(e);
	}
    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#updateCheck()
     */
    @Override
    protected void updateCheck() throws ApplicationException {

	// same as insertCheck
	this.insertCheck();
    }

}
