/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 *    https://progs.frankmuenster.de/eupl_v1-2_en/
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 ******************************************************************************/

package de.frankmuenster.jameica.depot.gui.input;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;

import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.input.AbstractInput;
import de.willuhn.jameica.gui.input.LabelInput;
import de.willuhn.jameica.gui.util.Color;
import de.willuhn.jameica.gui.util.SWTUtil;
import de.willuhn.jameica.system.OperationCanceledException;

/**
 * Das ist eine Erweiterung des {@link LabelInput} Objekts. Es kann nun ein
 * Tooltip sowie einen Rand haben. Durch setzten von Tooltip auf
 * <code>null</code> kann der Tooltip gel&ouml;scht werden.
 *
 * @author Frank Mnster
 */
public class LabelInputExtended extends AbstractInput {

    /**
     * Das AusgabeLabel
     */
    private transient Label label;

    /**
     * Der Labeltext
     */
    private transient String value;

    /**
     * Die Textfarbe
     */
    private transient Color color;

    /**
     * Das Text-Alignment
     */
    private int alignment;

    /**
     * Der Tooltiptext
     */
    private String toolTip;

    /**
     * Zeige Rahmen
     */
    private transient Boolean border;

    /**
     * Erzeugt ein neues Label mit dem angegebenen Wert.
     * 
     * @param value
     *            anzuzeigender Wert.
     */
    public LabelInputExtended(final String value) {
	this(value, null, false, SWT.LEFT);
    }

    /**
     * Erzeugt ein Label mit Wert, ohne Tooltip und Rand, falls
     * <code>border=true</code>.
     * 
     * @param value
     *            anzuzeigender Text
     * @param border
     *            mit Rand falls <code>true</code>, sonst ohne
     */
    public LabelInputExtended(final String value, final Boolean border) {
	this(value, null, border, SWT.LEFT);
    }

    /**
     * Erzeugt ein Label mit Wert, Rand, falls <code>border=true</code> und
     * Alignment
     * 
     * @param value
     *            anzuzeigender Text
     * @param border
     *            mit Rand falls <code>true</code>, sonst ohne
     * @param alignment
     *            das Alignment
     */
    public LabelInputExtended(final String value, final Boolean border, final int alignment) {
	this(value, null, border, alignment);
    }

    /**
     * Erzeugt ein Label mit Wert und Alignment.
     * 
     * @param value
     *            anzuzeigender Wert.
     * @param alignment
     *            das Alignment.
     */
    public LabelInputExtended(final String value, final int alignment) {
	this(value, null, false, alignment);
    }

    /**
     * Erzeugt ein Label mit Wert und Tooltip.
     * 
     * @param value
     *            anzuzeigender Wert.
     * @param tooltip
     *            anzuzeigender Tooltip
     */
    public LabelInputExtended(final String value, final String tooltip) {
	this(value, tooltip, false, SWT.LEFT);
    }

    /**
     * Erzeugt ein Label mit Wert, Tooltip, Rand, falls <code>border=true</code>
     * und Alignment.
     * 
     * @param value
     *            anzuzeigender Wert.
     * @param tooltip
     *            anzuzeigender Tooltip
     * @param border
     *            mit Rand falls <code>true</code>, sonst ohne
     * @param alignment
     *            das Alignment
     */
    public LabelInputExtended(final String value, final String tooltip, final Boolean border, final int alignment) {
	super();
	this.value = value;
	this.toolTip = tooltip;
	if (border == null) {
	    this.border = false;
	} else {
	    this.border = border;
	}
	this.alignment = alignment;
    }

    /**
     * Erzeugt ein Label mit Wert, Tooltip und Alignment.
     * 
     * @param value
     *            anzuzeigender Wert.
     * @param tooltip
     *            anzuzeigender Tooltip
     * @param alignment
     *            das Alignment
     */
    public LabelInputExtended(final String value, final String tooltip, final int alignment) {
	this(value, tooltip, false, alignment);
    }

    /**
     * @see de.willuhn.jameica.gui.input.Input#disable()
     */
    @Override
    public void disable() {
	// hier nix zu tun
    }

    /**
     * @see de.willuhn.jameica.gui.input.Input#enable()
     */
    @Override
    public void enable() {
	// hier nix zu tun
    }

    /**
     * @see de.willuhn.jameica.gui.input.Input#focus()
     */
    @Override
    public void focus() {
	// hier nix zu tun
    }

    /**
     * Liefert das Alignment.
     * 
     * @return das Alignment.
     */
    public int getAlignment() {
	return this.alignment;
    }

    /**
     * @see de.willuhn.jameica.gui.input.Input#getControl()
     */
    @Override
    public Control getControl() {
	if (this.border) {
	    this.label = GUI.getStyleFactory().createLabel(getParent(), SWT.BORDER);
	} else {
	    this.label = GUI.getStyleFactory().createLabel(getParent(), SWT.NONE);
	}
	this.label.setText(this.value == null ? "" : SWTUtil.escapeLabel(this.value)); //$NON-NLS-1$
	this.label.setToolTipText(this.toolTip == null ? null : SWTUtil.escapeLabel(this.toolTip));

	if (this.color != null) {
	    this.label.setForeground(this.color.getSWTColor());
	}

	this.label.setAlignment(this.alignment);

	return this.label;

    }

    /**
     * Liefert den Tooltip Text, oder null, falls kein Tooltip vorhanden ist.
     * 
     * @return den Tooltip Text.
     */
    public String getToolTip() {
	return this.toolTip;
    }

    /**
     * Liefert den angezeigten Text.
     * 
     * @see de.willuhn.jameica.gui.input.Input#getValue()
     */
    @Override
    public Object getValue() {
	return this.value;
    }

    /**
     * Liefert die Randeinstellung.
     * 
     * @return <code>true</code> falls das label einen Rand hat, sonst
     *         <code>false</code>.
     */
    public Boolean isBorder() {
	return this.border;
    }

    /**
     * @see de.willuhn.jameica.gui.input.Input#isEnabled()
     */
    @Override
    public boolean isEnabled() {
	return false;
    }

    /**
     * Legt das Alignment fest.
     * 
     * @param alignment
     *            das Alignment.
     */
    public void setAlignment(final int alignment) {
	this.alignment = alignment;
	if (this.label != null && !this.label.isDisposed()) {
	    this.label.setAlignment(alignment);
	    this.label.redraw();
	}
    }

    /**
     * Legt die Text-Farbe fest.
     * 
     * @param color
     *            Text-Farbe.
     */
    public void setColor(final Color color) {
	this.color = color;
	if (this.label != null && !this.label.isDisposed()) {
	    this.label.setForeground(this.color.getSWTColor());
	    this.label.redraw();
	}
    }

    /**
     * @see de.willuhn.jameica.gui.input.Input#setEnabled(boolean)
     */
    @Override
    public void setEnabled(final boolean enabled) {
	// Nee, das machen wir nicht
    }

    /**
     * @param tooltip
     *            Der Tooltiptext
     */
    public void setToolTip(final String tooltip) {

	this.toolTip = tooltip;
	if (this.label != null && !this.label.isDisposed()) {
	    this.label.setToolTipText(this.toolTip == null ? null : SWTUtil.escapeLabel(this.toolTip));
	    this.label.redraw();
	}
    }

    /**
     * Erwartet ein Objekt des Typs <code>java.lang.String</code>.
     * 
     * @see de.willuhn.jameica.gui.input.Input#setValue(java.lang.Object)
     */
    @Override
    public void setValue(final Object value) {
	if (value != null) {
	    this.value = (String) value;
	    if (this.label != null && !this.label.isDisposed()) {
		this.label.setText(SWTUtil.escapeLabel(value.toString()));
		this.label.redraw();
	    }
	}
    }

    /**
     * @see de.willuhn.jameica.gui.input.AbstractInput#update()
     */
    @Override
    protected void update() throws OperationCanceledException {
	// Wir machen hier nichts
    }

}
